__author__ = 'oriol'
import matplotlib
matplotlib.use('Agg')
from mpl_toolkits.basemap import Basemap  # , cm, maskoceans
import matplotlib.pyplot as plt
from scipy.interpolate import interp1d
import numpy as np
import os

home = os.path.expanduser("~")
directory=home+"/visualisation_frames"

if not os.path.exists(directory):
    os.makedirs(directory)

# Import data from netCDF part
'''
#  from netCDF4 import Dataset as NetCDFFile
#  import numpy.ma as ma
# nc = NetCDFFile('/home/oriol/PythonMaps/sst_1m.nc')
# tmpvar = nc.variables["sst_1d_ave"]
# data = tmpvar[0][:]
# lons = nc.variables['nav_lon'][:]
# lats = nc.variables['nav_lat'][:]
'''

#  Functions
'''
def abs(x):
    if x < 0:
        x=-x
    return x
'''
def interpolate(t, x, v, ti):
    """Given a set of points and the expected derivatives at those points,
    fit a cubic spline using the points and derivaties as constraints
    and return the interpolated values for each point in "ti"
    Input:
        t: The independent variable at each observed point
        x: The dependent variable at each observed point
        v: The derivate dx/dt at each observed point
        ti: The (new) t-values to solve for x at
    Output:
        xi: The interpolated x-values
    """
    # Note: there's probably a more elegant way of doing this...
    xi = np.zeros(ti.shape)
    for i in range(1, len(t)):
        t0, t1 = t[i-1], t[i]
        G = np.array([
            [t0**3, t0**2, t0, 1],
            [t1**3, t1**2, t1, 1],
            [3 * t0**2, 2 * t0, 1, 0],
            [3 * t1**2, 2 * t1, 1, 0],
            ])
        d = np.array([x[i-1], x[i], v[i-1], v[i]])
        m = np.linalg.solve(G,d)
        # One way of setting boundary conditions...
        if i == 1: # Points to the left of our data range
            interval = ti <= t[i]
        elif i == len(t)-1: # Points to the right of our data range
            interval = ti >= t[i-1]
        else: # Within bounds of data
            interval = (ti >= t[i-1]) & (ti <= t[i])
        t_int = ti[interval]
        xi[interval] = m[0] * t_int**3 + m[1] * t_int**2 + m[2] * t_int + m[3]
    return xi

def interpolate(t, x, v, ti):
    # Note: there's probably a more elegant way of doing this...
    xi = np.zeros(ti.shape)
    for i in range(1, len(t)):
        t0, t1 = t[i-1], t[i]
        G = np.array([
            [t0**3, t0**2, t0, 1],
            [t1**3, t1**2, t1, 1],
            [3 * t0**2, 2 * t0, 1, 0],
            [3 * t1**2, 2 * t1, 1, 0],
            ])
        d = np.array([x[i-1], x[i], v[i-1], v[i]])
        m = np.linalg.solve(G,d)
        # One way of setting boundary conditions...
        if i == 1: # Points to the left of our data range
            interval = ti <= t[i]
        elif i == len(t)-1: # Points to the right of our data range
            interval = ti >= t[i-1]
        else: # Within bounds of data
            interval = (ti >= t[i-1]) & (ti <= t[i])
        t_int = ti[interval]
        xi[interval] = m[0] * t_int**3 + m[1] * t_int**2 + m[2] * t_int + m[3]
    return xi


def fit_longitude(x, limit=360):
    while x > limit:
        x -= 360
    return x

def fit_latitude(x, limit=90):
    while abs(x) > limit:
        if x > limit:
            x = 2*limit - x
        elif x < -limit:
            x = -2*limit - x
    return x

def create_transition(lat_0, lon_0, lat_1, lon_1, transition_time=10.0, init_time=0.2, finish_time=0.2, fps=20.0):
    total_time = transition_time+init_time+finish_time
    t = [0, init_time, init_time+transition_time,
        init_time+transition_time+finish_time]
    lat = [lat_0, lat_0, lat_1, lat_1]
    lon = [lon_0, lon_0, lon_1, lon_1]
    tnew = np.linspace(0, total_time, total_time*fps)
    lat = interpolate(t, lat, [0, 0, 0, 0], tnew)
    lon = interpolate(t, lon, [0, 0, 0, 0], tnew)
    lat = map(fit_latitude, lat)
    lon = map(fit_longitude, lon)
    return tnew, lat, lon


latitudes = [42, -50, -40, 78, 42]
longitudes = [2, -60, -220, -200, -358]

frame = 0
total_frames = len(latitudes)*3*20
pi = 3.1415

for j in range(0, len(latitudes)-1):
    # Create smooth transition
    t, lat, lon = create_transition(latitudes[j], longitudes[j], latitudes[j+1], longitudes[j+1],
                                    init_time=0.5, transition_time=2, finish_time=0.5)

    for i in range(0, len(lon)):
        plt.figure(figsize=(10, 10))
        m = Basemap(projection='ortho', lon_0=lon[i], lat_0=lat[i],
        rsphere=6371200., resolution='l', area_thresh=10000)
        m.drawcoastlines()
        # m.bluemarble(scale=0.1)
        plt.savefig(directory+"/"+"transition" + format(frame, '04') + '.png')
        plt.clf()
        plt.close()
        print frame, "/", total_frames  # Print Progress
        frame += 1
        # plt.show()

# Plotting Data Part
'''
    # mdata = maskoceans(lons, lats, data, inlands=False, grid=1.25)
    # mask = ma.getmask(mdata)
    # mdata = ma.masked_array(data, np.logical_not(mask))
    # x, y = m(lons, lats)
    # x = ma.masked_array(x, np.logical_not(mask))
    # y = ma.masked_array(y, np.logical_not(mask))
    #  m.scatter(x, y, s=0.5, color='r', edgecolor="face", alpha=1)
    #  m.pcolor(x,y,mdata,edgecolor="face",alpha=1)
    #  m.pcolor(x,y,mdata,edgecolor="face",alpha=0.5/720*i)
'''
